searchData={"items":[{"type":"module","title":"AshJason.Resource","doc":"Ash resource extension for implementing `Jason.Encoder` protocol.","ref":"AshJason.Resource.html"},{"type":"macro","title":"AshJason.Resource.jason/1","doc":"","ref":"AshJason.Resource.html#jason/1"},{"type":"extras","title":"Guide","doc":"# AshJason\n\n[![Module Version](https://img.shields.io/hexpm/v/ash_jason)](https://hex.pm/packages/ash_jason)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen)](https://hexdocs.pm/ash_jason/)\n[![License](https://img.shields.io/hexpm/l/ash_jason)](https://github.com/vonagam/ash_jason/blob/master/LICENSE.md)\n\nAsh resource extension for implementing `Jason.Encoder` protocol.","ref":"readme.html"},{"type":"extras","title":"Installation - Guide","doc":"Add to the deps:\n\n```elixir\ndef deps do\n  [\n    {:ash_jason, \"~> 2.0.0\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Guide","doc":"Add `AshJason.Resource` to `extensions` list within `use Ash.Resource` options:\n\n```elixir\ndefmodule Example.Resource do\n  use Ash.Resource,\n    extensions: [AshJason.Resource]\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Configuration - Guide","doc":"Producing json object can have multiple steps:\n- Picking keys from a record.\n- Merging some values.\n- Renaming keys.\n- Ordering keys.\n- Customizing a result with a function.\n\nBy default only the picking step happens and it takes all non-private non-sensitive fields\n(attributes, relationships, aggregates, calculations) with loaded non-nil values from a record.\n\nFor adding and configuring those steps there is an optional `jason` dsl section:\n\n```elixir\ndefmodule Example.Resource do\n  use Ash.Resource,\n    extensions: [AshJason.Resource]\n\n  jason do\n    # options\n  end\nend\n```\n\nAll optional steps can be specified multiple times and are applied in the order they were defined in.\n\nA result object on which those steps operate is a key-value list - not map, not keyword list.\n- Unlike map the order is stable and guaranteed.\n- Unlike keyword list it can have string keys.\n\n#### `pick`\n\nKeys to pick from a record and include in the result.\nAccepts a fixed explicit list of keys or a map with a configuration of default behaviour.\n\nValues of `nil`/`Ash.NotLoaded`/`Ash.ForbiddenField` are omitted.\n\nMap can have such options as:\n- `private?` - Whenever to pick private fields.\n- `sensitive?` - Whenever to pick sensitive fields.\n- `include` - Keys to pick. In addition to fields.\n- `exclude` - Keys not to pick.\n\n```elixir\njason do\n  # Pick only those listed keys\n  pick [:only_some_field]\n\n  # Pick non-sensitive fields\n  pick %{private?: true}\n\n  # Pick non-private fields\n  pick %{sensitive?: true}\n\n  # Pick all fields\n  pick %{private?: true, sensitive?: true}\n\n  # Pick usual but include and exclude some specific keys\n  pick %{include: [:ok_private_field], exclude: [:irrelevant_public_field]}\nend\n```\n\n#### `merge`\n\nA step to merge values into a result.\nAccepts a map or a tuples list.\n\nMap has no guarantees about keys order so if you care about that prefer the list form.\n\n```elixir\njason do\n  # Merge with map\n  merge %{key: \"value\"}\n\n  # Merge with list\n  merge key: \"value\"\nend\n```\n\n#### `rename`\n\nA step to rename keys in a result.\nAccepts a map, a tuples list or a function for mapping.\n\n```elixir\njason do\n  # Rename with map\n  rename %{from_key: \"to_key\"}\n\n  # Rename with list\n  rename from_key: \"to_key\"\n\n  # Rename with a function\n  rename fn name -> String.capitalize(to_string(name)) end\nend\n```\n\n#### `order`\n\nA step to reorder keys in a result.\nAccepts a boolean, a sort function or a list of keys in a desired order.\n\nIf it is a list then it also acts as a filter and removes keys not present in that list.\n\n```elixir\njason do\n  # Order with standard `Enum.sort`\n  order true\n\n  # Order with a custom sort function\n  order fn keys -> Enum.sort(keys, :desc) end\n\n  # Order in accordance with a list\n  order [:only, :these, :keys, :in, :that, \"order\"]\nend\n```\n\n#### `customize`\n\nA step to arbitrary customize a result.\nAccepts a function that will get a result and a resource record as arguments and return a modifed result.\n\nAs mentioned above a result has a form of a list with two elements, key and value, tuples.\nTo work with it you might want to use `List` methods like `List.keytake` or `List.keystore`.\n\n```elixir\njason do\n  customize fn result, _record ->\n    result |> List.keystore(:custom_key, 0, {:custom_key, \"custom_value\"})\n  end\nend\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Links - Guide","doc":"[`Jason` docs](https://hexdocs.pm/jason).","ref":"readme.html#links"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2023 Dmitry Maganov\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"DSL","title":"jason - dsl-ashjason-resource","doc":"Configuration for Jason encoder implementation.","ref":"dsl-ashjason-resource.html#jason"},{"type":"DSL","title":"jason.pick - dsl-ashjason-resource","doc":"Keys to pick from a record into a result. Accepts an explicit names list or a behaviour configuration map.","ref":"dsl-ashjason-resource.html#jason-pick"},{"type":"DSL","title":"jason.merge - dsl-ashjason-resource","doc":"A step to merge fixed values into a result.","ref":"dsl-ashjason-resource.html#jason-merge"},{"type":"DSL","title":"jason.merge.values - dsl-ashjason-resource","doc":"Values to merge into a result. Accepts a map or a tuples list.","ref":"dsl-ashjason-resource.html#jason-merge-values"},{"type":"DSL","title":"jason.rename - dsl-ashjason-resource","doc":"A step to rename keys in a result.","ref":"dsl-ashjason-resource.html#jason-rename"},{"type":"DSL","title":"jason.rename.renames - dsl-ashjason-resource","doc":"A mapping for renaming keys in a result. Accepts a map, a tuples list or a function.","ref":"dsl-ashjason-resource.html#jason-rename-renames"},{"type":"DSL","title":"jason.order - dsl-ashjason-resource","doc":"A step to reorder keys in a result.","ref":"dsl-ashjason-resource.html#jason-order"},{"type":"DSL","title":"jason.order.sort - dsl-ashjason-resource","doc":"An order to apply to keys in json. Accepts a boolean, a sort function or a list of keys in a desired order.","ref":"dsl-ashjason-resource.html#jason-order-sort"},{"type":"DSL","title":"jason.customize - dsl-ashjason-resource","doc":"A step to arbitrary customize a result.","ref":"dsl-ashjason-resource.html#jason-customize"},{"type":"DSL","title":"jason.customize.fun - dsl-ashjason-resource","doc":"A function to customize a result with. Receives a result and a resource record.","ref":"dsl-ashjason-resource.html#jason-customize-fun"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}